@@ -22,7 +22,7 @@
 #    Contact : S..._Dot_I..._At_stkpp_Dot_org (see copyright for ...)
 #
 #-----------------------------------------------------------------------
-#' @include IClusterModel.R
+#' @include ClusterModelNames.R IClusterModel.R global.R
 NULL
 
 #-----------------------------------------------------------------------
@@ -38,7 +38,7 @@ NULL
 #' @param nbCluster  [\code{\link{vector}}] listing the number of clusters to test.
 #' @param modelNames [\code{\link{vector}}] of model names to run. By default all diagonal
 #' Gaussian models are estimated. All the model names are given by the method
-#' [\code{\link{diagGaussianNames}}].
+#' [\code{\link{clusterDiagGaussianNames}}].
 #' @param strategy a [\code{\linkS4class{ClusterStrategy}}] object containing
 #' the strategy to run. clusterStrategy() method by default.
 #' @param criterion character defining the criterion to select the best model.
@@ -65,13 +65,13 @@ NULL
 #' ## print model
 #' print(model)
 #' ## get estimated missing values
-#' model["data"][model["missings"]]
+#' missingValues(model)
 #'
 #' @return An instance of the [\code{\linkS4class{ClusterDiagGaussian}}] class.
 #' @author Serge Iovleff
 #' @export
 #'
-clusterDiagGaussian <- function(data, nbCluster=2, modelNames=diagGaussianNames(), strategy=clusterStrategy(), criterion="ICL")
+clusterDiagGaussian <- function(data, nbCluster=2, modelNames=clusterDiagGaussianNames(), strategy=clusterFastStrategy(), criterion="ICL")
 {
   # check nbCluster
   nbClusterModel = length(nbCluster);
@@ -89,47 +89,152 @@ clusterDiagGaussian <- function(data, nbCluster=2, modelNames=diagGaussianNames(
   if (ncol(data) < 1) {stop("Error: empty data set")}
 
   # check modelNames
-  if (!validDiagGaussianNames(modelNames))
-  { stop("modelNames is not valid. See ?diagGaussianNames for the list of valid model names")}
+  if (!clusterValidDiagGaussianNames(modelNames))
+  { stop("modelNames is not valid. See ?clusterDiagGaussianNames for the list of valid model names")}
 
   # check strategy
   if(class(strategy)[1] != "ClusterStrategy")
-  {stop("strategy is not a Cluster Stategy class (must be an instance of the class ClusterStrategy).")}
+  {stop("strategy is not a Cluster Strategy class (must be an instance of the class ClusterStrategy).")}
   validObject(strategy);
 
   # Create model
   model = new("ClusterDiagGaussian", data)
-  model@missings = which(is.na(data), arr.ind=TRUE);
   model@strategy = strategy;
   # start estimation of the models
   ResFlag <- FALSE;
-  res1Flag <- FALSE;
-  if (nbClusterMin == 1)
-  {
-    model = .diagGaussianNoCluster(model);
-    res1Flag <- TRUE;
-    ind <- which(nbCluster == 1, arr.ind = TRUE);
-    nbCluster <- nbCluster[-ind];
-  }
   if (length(nbCluster) >0)
   {
     resFlag = .Call("clusterMixture", model, nbCluster, modelNames, strategy, criterion, PACKAGE="rtkpp");
   }
   # set names
-  if (resFlag != TRUE && res1Flag != TRUE) {cat("WARNING: An error occur during the clustering process");}
-  else
-  {
-    colnames(model@mean)  <- colnames(model@data);
-    colnames(model@sigma) <- colnames(model@data);
-  }
+  if (resFlag != TRUE ) {cat("WARNING: An error occur during the clustering process");}
+  colnames(model@component@mean)  <- colnames(model@component@data);
+  colnames(model@component@sigma) <- colnames(model@component@data);
   model
 }
 
+#-----------------------------------------------------------------------
+#' Definition of the [\code{\linkS4class{ClusterDiagGaussianComponent}}] class
+#'
+#' This class defines a diagonal Gaussian component of a mixture Model.
+#'
+#' @slot mean  Matrix with the mean of the jth variable in the kth cluster.
+#' @slot sigma  Matrix with the standard deviation of the jth variable in the kth cluster.
+#' @seealso [\code{\linkS4class{IClusterComponent}}] class
+#'
+#' @examples
+#' getSlots("ClusterDiagGaussianComponent")
+#'
+#' @author Serge Iovleff
+#'
+#' @name ClusterDiagGaussianComponent
+#' @rdname IClusterComponent-class
+#' @aliases ClusterDiagGaussianComponent-class
+#' @exportClass ClusterDiagGaussianComponent
+#'
+setClass(
+  Class="ClusterDiagGaussianComponent",
+  representation( mean = "matrix", sigma = "matrix"),
+  contains=c("IClusterComponent"),
+  validity=function(object)
+  {
+    if (ncol(object@mean)!=ncol(object@data))
+    {stop("mean must have nbVariable columns.")}
+    if (ncol(object@sigma)!=ncol(object@data))
+    {stop("sigma must have nbVariable columns.")}
+    if (!clusterValidDiagGaussianNames(object@modelName))
+    {stop("Invalid Gaussian model name.")}
+    return(TRUE)
+  }
+)
+#' Initialize an instance of a rtkpp class.
+#'
+#' Initialization method of the [\code{\linkS4class{ClusterDiagGaussianComponent}}] class.
+#' Used internally in the `rtkpp' package.
+#'
+#' @rdname initialize-methods
+#' @keywords internal
+setMethod(
+    f="initialize",
+    signature=c("ClusterDiagGaussianComponent"),
+    definition=function(.Object, data=matrix(nrow=0, ncol=0), nbCluster=2, modelName="gaussian_pk_sjk")
+    {
+      # for data
+      if(missing(data)) {stop("data is mandatory in ClusterDiagGaussianComponent.")}
+      # check model name
+      if (!clusterValidDiagGaussianNames(modelName)) { stop("modelName is invalid");}
+      # call base class initialize
+      .Object <- callNextMethod(.Object, data, modelName)
+      # create slots
+      nbVariable = ncol(.Object@data);
+      .Object@mean  = matrix(0., nrow=nbCluster, ncol=nbVariable);
+      .Object@sigma = matrix(1., nrow=nbCluster, ncol=nbVariable);
+      # validate
+      validObject(.Object)
+      return(.Object)
+    }
+)
+
+#' @rdname extract-methods
+#' @aliases [,ClusterDiagGaussianComponent-method
+setMethod(
+    f="[",
+    signature(x = "ClusterDiagGaussianComponent"),
+    definition=function(x, i, j, drop)
+    {
+      if ( missing(j) )
+      {
+        switch(EXPR=i,
+            "mean" ={return(x@mean)},
+            "sigma"={return(x@sigma)},
+            stop("This attribute doesn't exist !")
+        )
+      }
+      else
+      {
+        if (!is.numeric(j)) {stop("j must be an integer.")}
+        if (round(j)!=j)    {stop("j must be an integer.")}
+        switch(EXPR=i,
+            "mean" ={return(x@mean[j,])},
+            "sigma"={return(x@sigma[j,])},
+            stop("This attribute doesn't exist !")
+        )
+      }
+    }
+)
+
+#' @rdname print-methods
+#' @aliases print print,ClusterDiagGaussianComponent-method
+#'
+setMethod(
+  signature=c("ClusterDiagGaussianComponent"),
+  f="print",
+  function(x,k,...)
+  {
+    cat("* Means     = ", format(x@mean[k,]), "\n")
+    cat("* S.D.      = ", format(x@sigma[k,]), "\n")
+  }
+)
+
+#' @rdname show-methods
+#' @aliases show-ClusterDiagGaussianComponent,ClusterDiagGaussianComponent,ClusterDiagGaussianComponent-method
+setMethod(
+  f="show",
+  signature=c("ClusterDiagGaussianComponent"),
+  function(object)
+  {
+    cat("* Means     = ", format(object@mean), "\n")
+    cat("* S.D.      = ", format(object@sigma), "\n")
+  }
+)
+
+
+#-----------------------------------------------------------------------
 #' Definition of the [\code{\linkS4class{ClusterDiagGaussian}}] class
 #'
 #' This class defines a diagonal Gaussian mixture Model.
 #'
-#' This class inherits from the [\code{\linkS4class{IClusterModel}}] class.
+#' This class inherits from the [\code{\linkS4class{IClusterModelBase}}] class.
 #' A diagonal gaussian model is a mixture model of the form:
 #' \deqn{
 #'   f({x}|\boldsymbol{\theta})
@@ -137,9 +242,9 @@ clusterDiagGaussian <- function(data, nbCluster=2, modelNames=diagGaussianNames(
 #'    \quad x \in {R}^d.
 #' }
 #'
-#' @slot mean  Matrix with the mean of the jth variable in the kth cluster.
-#' @slot sigma  Matrix with the standard deviation of the jth variable in the kth cluster.
-#' @seealso [\code{\linkS4class{IClusterModel}}] class
+#' @slot component  A [\code{\linkS4class{ClusterDiagGaussianComponent}}] with the
+#' mean and standard deviation of the diagonal mixture model.
+#' @seealso [\code{\linkS4class{IClusterModelBase}}] class
 #'
 #' @examples
 #' getSlots("ClusterDiagGaussian")
@@ -154,27 +259,25 @@ clusterDiagGaussian <- function(data, nbCluster=2, modelNames=diagGaussianNames(
 #' @exportClass ClusterDiagGaussian
 #'
 setClass(
-    Class="ClusterDiagGaussian",
-    representation( mean = "matrix", sigma = "matrix"),
-    contains=c("IClusterModel"),
-    prototype=list( mean   = matrix(nrow=0,ncol=0), sigma = matrix(nrow=0,ncol=0) ),
-    validity=function(object)
-    {
-      if (nrow(object@mean)!=object@nbCluster)
-      {stop("mean must have nbCluster rows.")}
-      if (ncol(object@mean)!=ncol(object@data))
-      {stop("mean must have nbVariable columns.")}
-      if (nrow(object@sigma)!=object@nbCluster)
-      {stop("sigma must have nbCluster rows.")}
-      if (ncol(object@sigma)!=ncol(object@data))
-      {stop("sigma must have nbVariable columns.")}
-      if (!validDiagGaussianNames(object@modelName))
-      {stop("Invalid Gaussian model name.")}
-      return(TRUE)
-    }
+  Class="ClusterDiagGaussian",
+  representation( component = "ClusterDiagGaussianComponent"),
+  contains=c("IClusterModelBase"),
+  validity=function(object)
+  {
+    if (nrow(object@component@mean)!=object@nbCluster)
+    {stop("mean must have nbCluster rows.")}
+    if (ncol(object@component@mean)!=ncol(object@component@data))
+    {stop("mean must have nbVariable columns.")}
+    if (nrow(object@component@sigma)!=object@nbCluster)
+    {stop("sigma must have nbCluster rows.")}
+    if (ncol(object@component@sigma)!=ncol(object@component@data))
+    {stop("sigma must have nbVariable columns.")}
+    if (!clusterValidDiagGaussianNames(object@component@modelName))
+    {stop("Invalid Gaussian model name.")}
+    return(TRUE)
+  }
 )
 
-#-----------------------------------------------------------------------
 #' Initialize an instance of a rtkpp class.
 #'
 #' Initialization method of the [\code{\linkS4class{ClusterDiagGaussian}}] class.
@@ -185,16 +288,13 @@ setClass(
 setMethod(
     f="initialize",
     signature=c("ClusterDiagGaussian"),
-    definition=function(.Object, data, nbCluster=2, modelName="gaussian_pk_sjk")
+    definition=function(.Object, data=matrix(nrow=0, ncol=0), nbCluster=2, modelName="gaussian_pk_sjk")
     {
-      .Object <- callNextMethod(.Object, data, nbCluster, modelName)
-      # resize
-      nbVariable <- ncol(.Object@data)
-      # for modelName
-      if(missing(modelName)) {.Object@modelName<-"gaussian_pk_sjk"}
-      else  {.Object@modelName<-modelName}
-      .Object@mean <- matrix(0, nbCluster, nbVariable)
-      .Object@sigma <- matrix(1, nbCluster, nbVariable)
+      # for data
+      if(missing(data)) {stop("data is mandatory in ClusterDiagGaussian.")}
+      # initialize component
+      .Object@component = new("ClusterDiagGaussianComponent", data, nbCluster, modelName);
+      .Object <- callNextMethod(.Object, nrow(.Object@component@data), nbCluster);
       # validate
       validObject(.Object)
       return(.Object)
@@ -209,14 +309,13 @@ setMethod(
   signature=c("ClusterDiagGaussian"),
   function(x,...){
     cat("****************************************\n")
-    callNextMethod()
+    callNextMethod();
     cat("****************************************\n")
     for(k in 1:length(x@pk))
     {
       cat("*** Cluster: ",k,"\n")
       cat("* Proportion = ", format(x@pk[k]), "\n")
-      cat("* Mean(s)    = ", format(x@mean[k,]), "\n")
-      cat("* Sd(s)      = ", format(x@sigma[k,]), "\n")
+      print(x@component, k);
       cat("****************************************\n")
     }
   }
@@ -225,36 +324,37 @@ setMethod(
 #' @rdname show-methods
 #' @aliases show-ClusterDiagGaussian,ClusterDiagGaussian,ClusterDiagGaussian-method
 setMethod(
-    f="show",
-    signature=c("ClusterDiagGaussian"),
-    function(object)
+  f="show",
+  signature=c("ClusterDiagGaussian"),
+  function(object)
+  {
+    cat("****************************************\n")
+    callNextMethod();
+    show(object@component);
+    cat("****************************************\n")
+    for(k in 1:length(object@pk))
     {
+      cat("*** Cluster: ",k,"\n")
+      cat("* Proportion = ", format(object@pk[k]), "\n")
+      print(object@component, k);
       cat("****************************************\n")
-      callNextMethod()
-      cat("****************************************\n")
-      for(k in 1:length(object@pk))
-      {
-        cat("*** Cluster: ",k,"\n")
-        cat("* Proportion = ", format(object@pk[k]), "\n")
-        cat("* Means      = ", format(object@mean[k,]), "\n")
-        cat("* Variances  = ", format(object@sigma[k,]), "\n")
-        cat("****************************************\n")
-      }
     }
+  }
 )
 
 #' @rdname summary-methods
 #' @aliases summary summary,ClusterDiagGaussian-method
 #'
 setMethod(
-    f="summary",
-    signature=c("ClusterDiagGaussian"),
-    function(object, ...)
-    {
-      cat("**************************************************************\n")
-      callNextMethod()
-      cat("**************************************************************\n")
-    }
+  f="summary",
+  signature=c("ClusterDiagGaussian"),
+  function(object, ...)
+  {
+    cat("**************************************************************\n")
+    callNextMethod()
+    summary(object@component);
+    cat("**************************************************************\n")
+  }
 )
 
 #' Plotting of a class [\code{\linkS4class{ClusterDiagGaussian}}]
@@ -296,24 +396,4 @@ setMethod(
 # wrapper of dnorm
 # x a vector with the point
 .dGauss <- function(x, j, k, model)
-{ dnorm(x, (model@mean)[k, j] , (model@sigma)[k, j])}
-
-# wrapper of dnorm
-# x a vector with the point
-.diagGaussianNoCluster <- function(model)
-{
-  nbSample   <- nrow(model@data);
-  nbVariable <- ncol(model@data);
-  model@mean  <- matrix(colMeans(model@data, na.rm = TRUE), nrow=1, ncol = nbVariable);
-  model@sigma <- matrix(apply(model@data, 2, na.rm = TRUE, sd), nrow=1, ncol = nbVariable);
-  model@nbCluster <- 1;
-  model@pk <- c(1);
-  model@tik <- matrix(1, nrow= nrow(model@data), ncol =1);
-  model@lnFi <- rowSums( t(apply(model@data, 1, mean=as.vector(model@mean), sd = as.vector(model@sigma), log = TRUE, dnorm)) );
-  model@zi <- as.integer(rep(1, nbVariable));
-  model@lnLikelihood <- sum(model@lnFi);
-  model@nbFreeParameter <- 2 * nbVariable;
-  model@criterion <- -2 * model@lnLikelihood + model@nbFreeParameter * log(nbSample);
-  model@modelName <- c("gaussian_pk_sjk");
-  model
-}
+{ dnorm(x, (model@component@mean)[k, j] , (model@component@sigma)[k, j])}
