@@ -40,43 +40,43 @@
 #' or \code{CEM} algorithms after initialization. This allow to detect "bad"
 #' initialization starting point of the estimation algorithm.
 #'
-#' These two stages are repeated until \code{nbInitRun} is reached. The initial
+#' These two stages are repeated until \code{nbInit} is reached. The initial
 #' point with the best log-likelihood is conserved as the initial starting point.
 #'
 #' @param method Character string with the initialisation method.
-#' Possible values: "random", "class", "fuzzy". Default value is "random".
+#' Possible values: "random", "class", "fuzzy". Default value is "class".
 #' @param algo Character string with the initialisation algorithm.
 #' Possible values: "EM", "CEM", "SEM", "SemiSEM". Default value is "SEM".
-#' @param nbInitRun integer defining the number of initialization point to test. Default value is 5.
+#' @param nbInit integer defining the number of initialization point to test. Default value is 5.
 #' @param nbIteration Integer defining the number of iteration in \code{algo}.
 #' nbIteration must be a positive integer. Default values is 20. Not used if  \code{algo} = NULL.
 #' @param epsilon Real defining the epsilon value for the algorithm. Default value: 0.01.
 #'
 #' @examples
-#'  clusterInit(method = "class", nbInitRun=1, algo="CEM",nbIteration=50, epsilon=0.00001)
+#'  clusterInit(method = "class", nbInit=1, algo="CEM",nbIteration=50, epsilon=0.00001)
 #'  clusterInit(nbIteration=0) # no algorithm
 #'
 #' @return a [\code{\linkS4class{ClusterInit}}] object
 #' @author Serge Iovleff
 #' @export
-clusterInit <- function( method="random", nbInitRun=5,  algo = "SEM", nbIteration=20, epsilon=0.01)
-{ return(new("ClusterInit", nbInitRun=nbInitRun, algo=new("ClusterAlgo", algo, nbIteration, epsilon)))}
+clusterInit <- function( method="class", nbInit=5,  algo = "SEM", nbIteration=20, epsilon=0.01)
+{ return(new("ClusterInit", nbInit=nbInit, algo=new("ClusterAlgo", algo, nbIteration, epsilon)))}
+
 
-#-----------------------------------------------------------------------
 #' Constructor of the [\code{\linkS4class{ClusterInit}}] class
 #'
 #' This class encapsulates the parameters of initialization methods of the
 #' rtkpp Cluster estimation method.
 #'
 #' @slot method Character string with the initialization method to use. Default value: "random"
-#' @slot nbInitRun Integer defining the number of initialization to perform. Default value: 5.
+#' @slot nbInit Integer defining the number of initialization to perform. Default value: 5.
 #' @slot algo An instance of \code{\linkS4class{ClusterAlgo}} class.
 #' Default value: \code{clusterAlgo("SEM", 20, 0)}.
 #'
 #' @examples
 #'   getSlots("ClusterInit")
 #'   new("ClusterInit")
-#'   new("ClusterInit", nbInitRun=1)
+#'   new("ClusterInit", nbInit=1)
 #'
 #' @author Serge Iovleff
 #'
@@ -86,34 +86,33 @@ clusterInit <- function( method="random", nbInitRun=5,  algo = "SEM", nbIteratio
 #' @exportClass ClusterInit
 #'
 setClass(
-    Class="ClusterInit",
-    slots=c(method="character", nbInitRun = "numeric", algo = "ClusterAlgo"),
-    prototype=list(method="random", nbInitRun = 5, algo = clusterAlgo("SEM", 20, 0)),
-    # validity function
-    validity=function(object)
+  Class="ClusterInit",
+  slots=c(method="character", nbInit = "numeric", algo = "ClusterAlgo"),
+  prototype=list(method="class", nbInit = 5, algo = clusterAlgo("SEM", 20, 0)),
+  # validity function
+  validity=function(object)
+  {
+    # for method
+    if ( sum(object@method %in% c("random","class","fuzzy")) != 1 )
+    {stop("Initialization method is not valid. See ?clusterInit for the list of available initialization method.")}
+    # for nbInit
+    if (round(object@nbInit)!=object@nbInit)
+    {stop("nbIInit must be an integer.")}
+    if( object@nbInit < 1 ) # can't be zero
+    {stop("nbInit must be strictly greater than 0.");}
+    # for algo
+    if (!is.null(object@algo))
     {
-      # for method
-      if ( sum(object@method %in% c("random","class","fuzzy")) != 1 )
-      {stop("Initialization method is not valid. See ?clusterInit for the list of available initialization method.")}
-      # for nbInitRun
-      if (round(object@nbInitRun)!=object@nbInitRun)
-      {stop("nbIInit must be an integer.")}
-      if( object@nbInitRun < 1 ) # can't be zero
-      {stop("nbInitRun must be strictly greater than 0.");}
-      # for algo
-      if (!is.null(object@algo))
-      {
-        if(class(object@algo)[1] != "ClusterAlgo")
-        {stop("algo is not of a Cluster algorithm (must be an instance of the class ClusterAlgo).")}
-        if (!validObject(object@algo))
-        {stop("algo is not of a valid algorithm. See ?clusterAlgo).")}
-      }
-      return(TRUE)
+      if(class(object@algo)[1] != "ClusterAlgo")
+      {stop("algo is not of a Cluster algorithm (must be an instance of the class ClusterAlgo).")}
+      if (!validObject(object@algo))
+      {stop("algo is not of a valid algorithm. See ?clusterAlgo).")}
     }
+    return(TRUE)
+  }
 )
 
 
-#-----------------------------------------------------------------------
 #' Initialize an instance of a rtkpp class.
 #'
 #' Initialization method of the [\code{\linkS4class{ClusterInit}}] class.
@@ -125,14 +124,14 @@ setClass(
 setMethod(
   f="initialize",
   signature=c("ClusterInit"),
-  definition=function(.Object,method="random",nbInitRun = 5,algo= clusterAlgo("SEM", 20, 0))
+  definition=function(.Object,method="class",nbInit = 5,algo= clusterAlgo("SEM", 20, 0))
   {
     # for method
-    if(missing(method)) {.Object@method<-"random"}
+    if(missing(method)) {.Object@method<-"class"}
     else  {.Object@method<-method}
     # for nbIteration
-    if( missing(nbInitRun) ){ .Object@nbInitRun<-5 }
-    else{.Object@nbInitRun<-nbInitRun}
+    if( missing(nbInit) ){ .Object@nbInit<-5 }
+    else{.Object@nbInit<-nbInit}
     # for algo
     if(missing(algo)){ .Object@algo<-clusterAlgo("SEM", 20, 0.1) }
     else{.Object@algo<-algo}
@@ -142,9 +141,6 @@ setMethod(
   }
 )
 
-#-----------------------------------------------------------------------
-# @name print
-# @docType methods
 #' @aliases print-init,ClusterInit,ClusterInit-method
 #' @rdname print-methods
 setMethod(
@@ -155,7 +151,7 @@ setMethod(
       cat("****************************************\n")
       cat("*** Cluster init:\n")
       cat("* method               = ", object@method, "\n")
-      cat("* number of init       = ", object@nbInitRun, "\n")
+      cat("* number of init       = ", object@nbInit, "\n")
       cat("* algorithm            = ", object@algo@algo, "\n")
       cat("* number of iterations = ", object@algo@nbIteration, "\n")
       cat("* epsilon              = ", object@algo@epsilon, "\n")
@@ -164,9 +160,6 @@ setMethod(
   }
 )
 
-#-----------------------------------------------------------------------
-# @name show
-# @docType methods
 #' @rdname show-methods
 #' @aliases show-init,ClusterInit,ClusterInit-method
 setMethod(
@@ -176,7 +169,7 @@ setMethod(
     cat("****************************************\n")
     cat("*** Cluster init:\n")
     cat("* method              = ", object@method, "\n")
-    cat("* number of init      = ", object@nbInitRun, "\n")
+    cat("* number of init      = ", object@nbInit, "\n")
     cat("* algorithm            = ", object@algo@algo, "\n")
     cat("* number of iterations = ", object@algo@nbIteration, "\n")
     cat("* epsilon              = ", object@algo@epsilon, "\n")
@@ -184,9 +177,6 @@ setMethod(
   }
 )
 
-#-----------------------------------------------------------------------
-# @name [
-# @docType methods
 #' @rdname extract-methods
 #' @aliases [,ClusterInit-method
 setMethod(
@@ -196,7 +186,7 @@ setMethod(
     if ( missing(j) ){
       switch(EXPR=i,
         "method"={return(x@method)},
-        "nbInitRun"={return(x@nbInitRun)},
+        "nbInit"={return(x@nbInit)},
         "algo"={return(x@algo)},
         stop("This attribute doesn't exist !")
         )
@@ -206,9 +196,7 @@ setMethod(
   }
 )
 
-#-----------------------------------------------------------------------
 #' @name [
-# @docType methods
 #' @rdname extract-methods
 #' @aliases [<-,ClusterInit-method
 setReplaceMethod(
@@ -219,7 +207,7 @@ setReplaceMethod(
     {
       switch(EXPR=i,
         "method"={x@method<-value},
-        "nbInitRun"={x@nbInitRun<-value},
+        "nbInit"={x@nbInit<-value},
         "algo"={x@algo<-value},
         stop("This attribute doesn't exist !")
       )
