@@ -214,9 +214,9 @@ namespace Arrays
 {
 
 /** @ingroup Arrays
- *  @brief Matrix sum (by coefficient)
- *  Perform the matrix sum coefficient by coefficient of the Matrix lhs
- *  by the Matrix rhs
+ *  @brief Array sum (by coefficient)
+ *  Perform the matrix sum coefficient by coefficient of the Array lhs
+ *  by the Array rhs
  **/
 template<typename Lhs, typename Rhs>
 struct SumOp
@@ -261,9 +261,9 @@ struct SumOp
 };
 
 /** @ingroup Arrays
- *  @brief Matrix difference (by coefficient)
- *  Perform the matrix difference coefficient by coefficient of the Matrix lhs
- *  by the Matrix rhs
+ *  @brief Array difference (by coefficient)
+ *  Perform the matrix difference coefficient by coefficient of the Array lhs
+ *  by the Array rhs
  **/
 template<typename Lhs, typename Rhs>
 struct DifferenceOp
@@ -308,9 +308,9 @@ struct DifferenceOp
 };
 
 /** @ingroup Arrays
- *  @brief Matrix product (by coefficient)
- *  Perform the matrix product coefficient by coefficient of the Matrix lhs
- *  by the Matrix rhs
+ *  @brief Array product (by coefficient)
+ *  Perform the matrix product coefficient by coefficient of the Array lhs
+ *  by the Array rhs
  **/
 template<typename Lhs, typename Rhs>
 struct Product
@@ -349,8 +349,8 @@ struct Product
 /** @ingroup Arrays
  *  @brief Matricial Division
  *
- *  Perform the matrix division coefficient by coefficient of the Matrix lhs
- *  by the Matrix rhs
+ *  Perform the matrix division coefficient by coefficient of the Array lhs
+ *  by the Array rhs
  **/
 template<typename Lhs, typename Rhs>
 struct DivOp
@@ -387,8 +387,8 @@ struct DivOp
 };
 
 /** @ingroup Arrays
- *  @brief Matrix multiplication
- *  Perform the matrix product of the Matrix lhs by the Matrix rhs
+ *  @brief Array multiplication
+ *  Perform the matrix product of the Array lhs by the Array rhs
  **/
 template<typename Lhs, typename Rhs>
 struct MultOp
@@ -441,8 +441,8 @@ struct MultOp
 };
 
 /** @ingroup Arrays
- *  @brief Matrix multiplication
- *  Perform the matrix product of the transposed Matrix lhs by the Matrix rhs
+ *  @brief Array multiplication
+ *  Perform the matrix product of the transposed Array lhs by the Array rhs
  **/
 template<typename Lhs, typename Rhs>
 struct MultLeftTransposeOp
@@ -493,8 +493,8 @@ struct MultLeftTransposeOp
 };
 
 /** @ingroup Arrays
- *  @brief Matrix multiplication
- *  Perform the matrix product of the Matrix lhs by the transposed Matrix rhs
+ *  @brief Array multiplication
+ *  Perform the matrix product of the Array lhs by the transposed Array rhs
  **/
 template<typename Lhs, typename Rhs>
 struct MultRightTransposeOp
@@ -629,8 +629,8 @@ inline typename Arrays::MultRightTransposeOp<Lhs, Rhs>::result_type
  *  @return the dot product of the two vectors
  **/
 template<class Container1D1, class Container1D2>
-Real dot( ITContainer< Container1D1> const& x
-        , ITContainer< Container1D2> const& y
+Real dot( ExprBase< Container1D1> const& x
+        , ExprBase< Container1D2> const& y
         )
 {
   // compute the valid range
@@ -661,17 +661,16 @@ Real dot( ITContainer< Container1D1> const& x
  *  @return the weighted dot product of the two vectors
  **/
 template<class Container1D1, class Container1D2, class Container1D3>
-Real weightedDot( ITContainer< Container1D1> const& x
-                , ITContainer< Container1D2> const& y
-                , ITContainer< Container1D3> const& w
+Real weightedDot( ExprBase< Container1D1> const& x
+                , ExprBase< Container1D2> const& y
+                , ExprBase< Container1D3> const& w
                 )
 {
   // compute the valid range
   const int first = std::max(x.begin(), y.begin()) , last = std::min(x.lastIdx(), y.lastIdx());
 #ifdef STK_DEBUG
-  if (!Range(first,last).isIn(w.range()))
-    throw runtime_error("In weightedDot(x, w) "
-                             "Range(first,last) not include in w.range()");
+  if (!Range(first,last,0).isIn(w.range()))
+  { STKRUNTIME_ERROR_2ARG(In weightedDot(x,y,w),Range(first,last,0),w.range(),first:last not include in w.range());}
 #endif
   // compute the sum product
   Real sum=0.0;
@@ -697,9 +696,7 @@ Real weightedDot( ITContainer< Container1D1> const& x
  *  @return the Euclidian distance between x and y
  **/
 template<class Container1D1, class Container1D2>
-Real dist( ITContainer< Container1D1> const& x
-         , ITContainer< Container1D2> const& y
-         )
+Real dist( ExprBase< Container1D1> const& x, ExprBase< Container1D2> const& y)
 {
   // compute the valid range
   const int first = std::max(x.begin(), y.begin()) , last = std::min(x.lastIdx(), y.lastIdx());
@@ -736,9 +733,9 @@ Real dist( ITContainer< Container1D1> const& x
  *  @return the weighted Euclidian distance between x and y
  **/
 template<class Container1D1, class Container1D2, class Container1D3>
-Real weightedDist( ITContainer< Container1D1> const& x
-                 , ITContainer< Container1D2> const& y
-                 , ITContainer< Container1D3> const& w
+Real weightedDist( ExprBase< Container1D1> const& x
+                 , ExprBase< Container1D2> const& y
+                 , ExprBase< Container1D3> const& w
                  )
 {
   // compute the valid range
@@ -766,7 +763,7 @@ Real weightedDist( ITContainer< Container1D1> const& x
 }
 
 /** @ingroup Arrays
- *  @brief Matrix multiplication by its transpose
+ *  @brief Array multiplication by its transpose
  *
  *  Perform the matrix product \f$ A'A \f$.
  *
@@ -793,7 +790,7 @@ Array2DSquare<typename Derived::Type> multLeftTranspose( ExprBase<Derived> const
 }
 
 /** @ingroup Arrays
- *  @brief Matrix multiplication by its transpose
+ *  @brief Array multiplication by its transpose
  *
  *  Perform the matrix product \f$ AA' \f$.
  *
@@ -851,7 +848,7 @@ Array2DSquare<typename Derived::Type>
 }
 
 /** @ingroup Arrays
- *  @brief Weigthed Matrix multiplication by its transpose
+ *  @brief weighted Array multiplication by its transpose
  *
  *  Perform the matrix product \f$ AWA' \f$.
  *
