@@ -53,7 +53,6 @@ struct MixtureTraits< Gamma_a_bjk<_Array> >
 {
   typedef _Array Array;
   typedef typename Array::Type Type;
-  typedef MixtureComponent<_Array, Gamma_a_bjk_Parameters> Component;
   typedef Gamma_a_bjk_Parameters        Parameters;
   typedef Array2D<Real>        Param;
 };
@@ -73,25 +72,25 @@ template<class Array>
 class Gamma_a_bjk : public GammaBase< Gamma_a_bjk<Array> >
 {
   public:
-    typedef typename Clust::MixtureTraits< Gamma_a_bjk<Array> >::Component Component;
     typedef typename Clust::MixtureTraits< Gamma_a_bjk<Array> >::Parameters Parameters;
     typedef GammaBase< Gamma_a_bjk<Array> > Base;
 
-    using Base::p_tik;
+    using Base::p_tik;using Base::components;
     using Base::p_data;
     using Base::p_param;
-    using Base::components;
+
     using Base::meanjk;
     using Base::variancejk;
 
     /** default constructor
      * @param nbCluster number of cluster in the model
      **/
-    inline Gamma_a_bjk( int nbCluster) : Base(nbCluster), shape_(1) {}
+    inline Gamma_a_bjk( int nbCluster) : Base(nbCluster), shape_(1), stat_shape_() {}
     /** copy constructor
      *  @param model The model to copy
      **/
-    inline Gamma_a_bjk( Gamma_a_bjk const& model) : Base(model), shape_(model.shape_) {}
+    inline Gamma_a_bjk( Gamma_a_bjk const& model)
+                      : Base(model), shape_(model.shape_), stat_shape_(model.stat_shape_) {}
     /** destructor */
     inline ~Gamma_a_bjk() {}
     /** Initialize the component of the model.
@@ -103,9 +102,24 @@ class Gamma_a_bjk : public GammaBase< Gamma_a_bjk<Array> >
       shape_ = 1.;
       for (int k= baseIdx; k < components().end(); ++k)
       { p_param(k)->p_shape_ = &shape_;}
+      stat_shape_.initialize();
+    }
+    /** Store the intermediate results of the Mixture.
+     *  @param iteration Provides the iteration number beginning after the burn-in period.
+     **/
+    void storeIntermediateResultsImpl(int iteration)
+    { stat_shape_.update(shape_);}
+    /** Release the stored results. This is usually used if the estimation
+     *  process failed.
+     **/
+    void releaseIntermediateResultsImpl()
+    { stat_shape_.release();}
+    /** set the parameters stored in stat_proba_ and release stat_proba_. */
+    void setParametersImpl()
+    {
+      shape_ = stat_shape_.param_;
+      stat_shape_.release();
     }
-    /** initialize shape and scale parameters using weighted moment estimates.*/
-    inline bool initializeStep() { return mStep();}
     /** Initialize randomly the parameters of the Gamma mixture. The shape
      *  will be selected randomly using an exponential of parameter mean^2/variance
      *  and the scale will be selected randomly using an exponential of parameter
@@ -121,6 +135,8 @@ class Gamma_a_bjk : public GammaBase< Gamma_a_bjk<Array> >
   protected:
     /** common shape */
     Real shape_;
+    /** Satistics on the shape */
+    MixtureStatReal stat_shape_;
 };
 
 template<class Array>
@@ -155,7 +171,7 @@ bool Gamma_a_bjk<Array>::mStep()
   Real y =0.0, x0 = 0.0, x1 = shape_;
   for (int j=p_data()->beginCols(); j < p_data()->endCols(); ++j)
   {
-    for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+    for (int k= baseIdx; k < components().end(); ++k)
     {
       Real mean = meanjk(j,k);
       y  += p_param(k)->tk_ * (p_param(k)->meanLog_[j]-std::log(mean));
@@ -165,7 +181,7 @@ bool Gamma_a_bjk<Array>::mStep()
   y  /= (this->nbSample()*this->nbVariable());
   x0 /= (this->nbSample()*this->nbVariable());
   // moment estimate and oldest value
-  if ((x0 <=0.) || (Arithmetic<Real>::isNA(x0))) return false;
+  if ((x0 <=0.) || (isNA(x0))) return false;
 
   // get shape
   hidden::invPsiMLog f(y);
@@ -186,7 +202,7 @@ bool Gamma_a_bjk<Array>::mStep()
   // estimate bjk
   for (int j=p_data()->beginCols(); j < p_data()->endCols(); ++j)
   {
-    for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+    for (int k= baseIdx; k < components().end(); ++k)
     { p_param(k)->scale_[j] = p_param(k)->mean_[j]/a;}
   }
   return true;
