@@ -53,7 +53,6 @@ struct MixtureTraits< Gamma_a_bk<_Array> >
 {
   typedef _Array Array;
   typedef typename Array::Type Type;
-  typedef MixtureComponent<_Array, Gamma_a_bk_Parameters> Component;
   typedef Gamma_a_bk_Parameters        Parameters;
   typedef Array2D<Real>        Param;
 };
@@ -73,25 +72,25 @@ template<class Array>
 class Gamma_a_bk : public GammaBase< Gamma_a_bk<Array> >
 {
   public:
-    typedef typename Clust::MixtureTraits< Gamma_a_bk<Array> >::Component Component;
     typedef typename Clust::MixtureTraits< Gamma_a_bk<Array> >::Parameters Parameters;
     typedef GammaBase< Gamma_a_bk<Array> > Base;
 
-    using Base::p_tik;
+     using Base::p_tik;using Base::components;
     using Base::p_data;
     using Base::p_param;
-    using Base::components;
+
     using Base::meanjk;
     using Base::variancejk;
 
     /** default constructor
      * @param nbCluster number of cluster in the model
      **/
-    inline Gamma_a_bk( int nbCluster) : Base(nbCluster), shape_(1) {}
+    inline Gamma_a_bk( int nbCluster) : Base(nbCluster), shape_(1),stat_shape_() {}
     /** copy constructor
      *  @param model The model to copy
      **/
-    inline Gamma_a_bk( Gamma_a_bk const& model) : Base(model), shape_(model.shape_) {}
+    inline Gamma_a_bk( Gamma_a_bk const& model)
+                     : Base(model), shape_(model.shape_),stat_shape_(model.stat_shape_) {}
     /** destructor */
     inline ~Gamma_a_bk() {}
     /** Initialize the component of the model.
@@ -104,8 +103,22 @@ class Gamma_a_bk : public GammaBase< Gamma_a_bk<Array> >
       for (int k= baseIdx; k < components().end(); ++k)
       { p_param(k)->p_shape_ = &shape_;}
     }
-    /** initialize shape and scale parameters using weighted moment estimates.*/
-    inline bool initializeStep() { return mStep();}
+    /** Store the intermediate results of the Mixture.
+     *  @param iteration Provides the iteration number beginning after the burn-in period.
+     **/
+    void storeIntermediateResultsImpl(int iteration)
+    { stat_shape_.update(shape_);}
+    /** Release the stored results. This is usually used if the estimation
+     *  process failed.
+     **/
+    void releaseIntermediateResultsImpl()
+    { stat_shape_.release();}
+    /** set the parameters stored in stat_proba_ and release stat_proba_. */
+    void setParametersImpl()
+    {
+      shape_ = stat_shape_.param_;
+      stat_shape_.release();
+    }
     /** Initialize randomly the parameters of the Gamma mixture. The shape
      *  will be selected randomly using an exponential of parameter mean^2/variance
      *  and the scale will be selected randomly using an exponential of parameter
@@ -115,19 +128,20 @@ class Gamma_a_bk : public GammaBase< Gamma_a_bk<Array> >
     /** Compute the mStep. */
     bool mStep();
     /** @return the number of free parameters of the model */
-    inline int computeNbFreeParameters() const
-    { return this->nbCluster()+1;}
+    inline int computeNbFreeParameters() const { return this->nbCluster()+1;}
 
   protected:
     /** common scale */
     Real shape_;
+    /** statistics */
+    MixtureStatReal stat_shape_;
 };
 
 template<class Array>
 void Gamma_a_bk<Array>::randomInit()
 {
-    // compute moments
-    this->moments();
+  // compute moments
+  this->moments();
   // generate scales
   Real value = 0.0;
   for (int k= baseIdx; k < components().end(); ++k)
@@ -160,7 +174,7 @@ bool Gamma_a_bk<Array>::mStep()
   y  /= (this->nbSample()*this->nbVariable());
   x0 /= this->nbSample();
   // moment estimate and oldest value
-  if ((x0 <=0.) || (Arithmetic<Real>::isNA(x0))) return false;
+  if ((x0 <=0.) || (isNA(x0))) return false;
 
   // get shape
   hidden::invPsiMLog f(y);
@@ -179,7 +193,7 @@ bool Gamma_a_bk<Array>::mStep()
   // set values
   shape_ = a;
   // estimate b
-  for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+  for (int k= baseIdx; k < components().end(); ++k)
   { p_param(k)->scale_ = this->meank(k)/a;}
   return true;
 }
