@@ -55,7 +55,6 @@ struct MixtureTraits< Gamma_aj_bk<_Array> >
 {
   typedef _Array Array;
   typedef typename Array::Type Type;
-  typedef MixtureComponent<_Array, Gamma_aj_bk_Parameters> Component;
   typedef Gamma_aj_bk_Parameters        Parameters;
   typedef Array2D<Real>        Param;
 };
@@ -75,25 +74,26 @@ template<class Array>
 class Gamma_aj_bk : public GammaBase< Gamma_aj_bk<Array> >
 {
   public:
-    typedef typename Clust::MixtureTraits< Gamma_aj_bk<Array> >::Component Component;
     typedef typename Clust::MixtureTraits< Gamma_aj_bk<Array> >::Parameters Parameters;
     typedef GammaBase< Gamma_aj_bk<Array> > Base;
 
     using Base::p_tik;
+    using Base::components;
     using Base::p_data;
     using Base::p_param;
-    using Base::components;
+
     using Base::meanjk;
     using Base::variancejk;
 
     /** default constructor
      * @param nbCluster number of cluster in the model
      **/
-    inline Gamma_aj_bk( int nbCluster) : Base(nbCluster), shape_() {}
+    inline Gamma_aj_bk( int nbCluster) : Base(nbCluster), shape_(), stat_shape_() {}
     /** copy constructor
      *  @param model The model to copy
      **/
-    inline Gamma_aj_bk( Gamma_aj_bk const& model) : Base(model), shape_(model.shape_) {}
+    inline Gamma_aj_bk( Gamma_aj_bk const& model)
+                      : Base(model), shape_(model.shape_), stat_shape_(model.stat_shape_) {}
     /** destructor */
     inline ~Gamma_aj_bk() {}
     /** Initialize the component of the model.
@@ -106,9 +106,24 @@ class Gamma_aj_bk : public GammaBase< Gamma_aj_bk<Array> >
       shape_ = 1.;
       for (int k= baseIdx; k < components().end(); ++k)
       { p_param(k)->p_shape_ = &shape_;}
+      stat_shape_.initialize(p_data()->cols());
+    }
+    /** Store the intermediate results of the Mixture.
+     *  @param iteration Provides the iteration number beginning after the burn-in period.
+     **/
+    void storeIntermediateResultsImpl(int iteration)
+    { stat_shape_.update(shape_);}
+    /** Release the stored results. This is usually used if the estimation
+     *  process failed.
+     **/
+    void releaseIntermediateResultsImpl()
+    { stat_shape_.release();}
+    /** set the parameters stored in stat_proba_ and release stat_proba_. */
+    void setParametersImpl()
+    {
+      shape_ = stat_shape_.param_;
+      stat_shape_.release();
     }
-    /** initialize shape and scale parameters using weighted moment estimates.*/
-    inline bool initializeStep() { return mStep();};
     /** Initialize randomly the parameters of the Gamma mixture. The shape
      *  will be selected randomly using an exponential of parameter mean^2/variance
      *  and the scale will be selected randomly using an exponential of parameter
@@ -122,8 +137,10 @@ class Gamma_aj_bk : public GammaBase< Gamma_aj_bk<Array> >
     { return(this->nbCluster()+this->nbVariable());}
 
   protected:
-    /** common shape */
+    /** common shapes */
     Array2DPoint<Real> shape_;
+    /** Statistics of the common shape */
+    MixtureStatVector stat_shape_;
 };
 
 /* Initialize randomly the parameters of the gamma mixture. The centers
@@ -171,7 +188,7 @@ bool Gamma_aj_bk<Array>::mStep()
     {
       // moment estimate and oldest value
       Real y = 0, x0 = 0;
-      for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+      for (int k= baseIdx; k < components().end(); ++k)
       {
         Real mean = meanjk(j,k), variance = variancejk(j,k);
         y  += p_param(k)->tk_ * (p_param(k)->meanLog_[j] - std::log(p_param(k)->scale_));
@@ -199,7 +216,7 @@ bool Gamma_aj_bk<Array>::mStep()
       else { shape_[j] = a;}
       // compute bk
       Real sum = shape_.sum();
-      for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+      for (int k= baseIdx; k < components().end(); ++k)
       { // update bk
         p_param(k)->scale_ = p_param(k)->mean_.sum()/sum;
       }
