@@ -53,7 +53,6 @@ struct MixtureTraits< Gamma_ajk_bjk<_Array> >
 {
   typedef _Array Array;
   typedef typename Array::Type Type;
-  typedef MixtureComponent<_Array, Gamma_ajk_bjk_Parameters> Component;
   typedef Gamma_ajk_bjk_Parameters        Parameters;
   typedef Array2D<Real>        Param;
 };
@@ -73,14 +72,14 @@ template<class Array>
 class Gamma_ajk_bjk : public GammaBase< Gamma_ajk_bjk<Array> >
 {
   public:
-    typedef typename Clust::MixtureTraits< Gamma_ajk_bjk<Array> >::Component Component;
     typedef typename Clust::MixtureTraits< Gamma_ajk_bjk<Array> >::Parameters Parameters;
     typedef GammaBase< Gamma_ajk_bjk<Array> > Base;
 
     using Base::p_tik;
+    using Base::components;
     using Base::p_data;
     using Base::p_param;
-    using Base::components;
+
     using Base::meanjk;
     using Base::variancejk;
 
@@ -94,8 +93,8 @@ class Gamma_ajk_bjk : public GammaBase< Gamma_ajk_bjk<Array> >
     inline Gamma_ajk_bjk( Gamma_ajk_bjk const& model) : Base(model) {}
     /** destructor */
     inline ~Gamma_ajk_bjk() {}
-    /** initialize shape and scale parameters using weighted moment estimates.*/
-    inline bool initializeStep() { return mStep();}
+    /** Initialize the model. */
+    void initializeModelImpl() {}
     /** Initialize randomly the parameters of the Gamma mixture. The shape
      *  will be selected randomly using an exponential of parameter mean^2/variance
      *  and the scale will be selected randomly using an exponential of parameter
@@ -139,14 +138,14 @@ bool Gamma_ajk_bjk<Array>::mStep()
 {
   if (!this->moments()) { return false;}
   // estimate a and b
-  for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+  for (int k= baseIdx; k < components().end(); ++k)
   {
     for (int j=p_data()->beginCols(); j < p_data()->endCols(); ++j)
     {
       // moment estimate and oldest value
       Real x0 = meanjk(j,k)*meanjk(j,k)/variancejk(j,k);
       Real x1 = p_param(k)->shape_[j];
-      if ((x0 <=0.) || (Arithmetic<Real>::isNA(x0))) return false;
+      if ((x0 <=0.) || (isNA(x0))) return false;
 
       // get shape
       hidden::invPsiMLog f(p_param(k)->meanLog_[j]-std::log(p_param(k)->mean_[j]));
