@@ -56,7 +56,6 @@ struct MixtureTraits< Gamma_ak_bj<_Array> >
   typedef _Array Array;
   typedef typename Array::Type Type;
   typedef Gamma_ak_bj_Parameters Parameters;
-  typedef MixtureComponent<_Array, Parameters> Component;
   typedef Array2D<Real>        Param;
 };
 
@@ -75,27 +74,26 @@ template<class Array>
 class Gamma_ak_bj : public GammaBase<Gamma_ak_bj<Array> >
 {
   public:
-    typedef typename Clust::MixtureTraits< Gamma_ak_bj<Array> >::Component Component;
     typedef typename Clust::MixtureTraits< Gamma_ak_bj<Array> >::Parameters Parameters;
     typedef GammaBase<Gamma_ak_bj<Array> > Base;
 
-    typedef Array2D<Real>::ColVector ColVector;
-
     using Base::p_tik;
+    using Base::components;
     using Base::p_data;
     using Base::p_param;
-    using Base::components;
+
     using Base::meanjk;
     using Base::variancejk;
 
     /** default constructor
      * @param nbCluster number of cluster in the model
      **/
-    inline Gamma_ak_bj( int nbCluster) : Base(nbCluster), scale_() {}
+    inline Gamma_ak_bj( int nbCluster) : Base(nbCluster), scale_(), stat_scale_() {}
     /** copy constructor
      *  @param model The model to copy
      **/
-    inline Gamma_ak_bj( Gamma_ak_bj const& model) : Base(model), scale_(model.scale_) {}
+    inline Gamma_ak_bj( Gamma_ak_bj const& model)
+                      : Base(model), scale_(model.scale_), stat_scale_(model.stat_scale_) {}
     /** destructor */
     inline ~Gamma_ak_bj() {}
     /** Initialize the component of the model.
@@ -104,15 +102,28 @@ class Gamma_ak_bj : public GammaBase<Gamma_ak_bj<Array> >
      **/
     void initializeModelImpl()
     {
-      scale_.resize(this->nbVariable());
+      scale_.resize(p_data()->cols());
       scale_ = 1.;
       for (int k= baseIdx; k < components().end(); ++k)
       { p_param(k)->p_scale_ = &scale_;}
+      stat_scale_.initialize(p_data()->cols());
     }
-    /** use the default static method initializeStep() for a first initialization
-     *  of the parameters using tik values.
+    /** Store the intermediate results of the Mixture.
+     *  @param iteration Provides the iteration number beginning after the burn-in period.
      **/
-    inline bool initializeStep() { return mStep();}
+    void storeIntermediateResultsImpl(int iteration)
+    { stat_scale_.update(scale_);}
+    /** Release the stored results. This is usually used if the estimation
+     *  process failed.
+     **/
+    void releaseIntermediateResultsImpl()
+    { stat_scale_.release();}
+    /** set the parameters stored in stat_proba_ and release stat_proba_. */
+    void setParametersImpl()
+    {
+      scale_ = stat_scale_.param_;
+      stat_scale_.release();
+    }
     /** Initialize randomly the parameters of the Gaussian mixture. The centers
      *  will be selected randomly among the data set and the standard-deviation
      *  will be set to 1.
@@ -127,6 +138,8 @@ class Gamma_ak_bj : public GammaBase<Gamma_ak_bj<Array> >
   protected:
     /** Array of the common scale */
     Array2DPoint<Real> scale_;
+    /** Vector of the statistics */
+    MixtureStatVector stat_scale_;
 };
 
 /* Initialize randomly the parameters of the Gaussian mixture. The centers
@@ -177,7 +190,7 @@ bool Gamma_ak_bj<Array>::mStep()
   for(iter=0; iter<MAXITER; ++iter)
   {
     // compute ak
-    for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+    for (int k= baseIdx; k < components().end(); ++k)
     {
       // moment estimate and oldest value
       Real x0 = (p_param(k)->mean_.square()/p_param(k)->variance_).mean();
@@ -205,7 +218,7 @@ bool Gamma_ak_bj<Array>::mStep()
     for (int j=p_data()->beginCols(); j<p_data()->endCols(); ++j)
     {
       Real num = 0., den = 0.;
-      for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+      for (int k= baseIdx; k < components().end(); ++k)
       {
         num += p_param(k)->mean_[j] * p_param(k)->tk_;
         den += p_param(k)->shape_   * p_param(k)->tk_;
