@@ -53,7 +53,6 @@ struct MixtureTraits< Gaussian_sk<_Array> >
   typedef _Array Array;
   typedef typename Array::Type Type;
   typedef Gaussian_sk_Parameters Parameters;
-  typedef MixtureComponent<_Array, Parameters> Component;
   typedef Array2D<Real>        Param;
 };
 
@@ -73,15 +72,14 @@ class Gaussian_sk : public DiagGaussianBase<Gaussian_sk<Array> >
 {
   public:
     typedef DiagGaussianBase<Gaussian_sk<Array> > Base;
-    typedef typename Clust::MixtureTraits< Gaussian_sk<Array> >::Component Component;
     typedef typename Clust::MixtureTraits< Gaussian_sk<Array> >::Parameters Parameters;
 
-    typedef Array2D<Real>::ColVector ColVector;
-
     using Base::p_tik;
+    using Base::p_nk;
+    using Base::components;
     using Base::p_data;
     using Base::p_param;
-    using Base::components;
+
 
     /** default constructor
      * @param nbCluster number of cluster in the model
@@ -93,8 +91,6 @@ class Gaussian_sk : public DiagGaussianBase<Gaussian_sk<Array> >
     inline Gaussian_sk( Gaussian_sk const& model) : Base(model) {}
     /** destructor */
     inline ~Gaussian_sk() {}
-    /** Compute the inital weighted mean and the initial common standard deviations. */
-    inline bool initializeStep() { return mStep();}
     /** Initialize randomly the parameters of the Gaussian mixture. The centers
      *  will be selected randomly among the data set and the standard-deviations
      *  will be set to 1.
@@ -117,12 +113,12 @@ void Gaussian_sk<Array>::randomInit()
   this->randomMean();
   // compute the standard deviation
   Real variance;
-  for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+  for (int k= baseIdx; k < components().end(); ++k)
   {
     variance = sqrt( ( p_tik()->col(k).transpose()
                      *(*p_data() - (Const::Vector<Real>(p_data()->rows()) * p_param(k)->mean_)
                       ).square()
-                     ).sum() / (p_data()->sizeCols()*p_tik()->col(k).sum())
+                     ).sum() / (p_data()->sizeCols()*p_nk()->elt(k))
                    );
     p_param(k)->sigma_ = ((variance<=0) || !Arithmetic<Real>::isFinite(variance))
                        ? 1.
@@ -140,7 +136,7 @@ bool Gaussian_sk<Array>::mStep()
   // compute the means
   if (!this->updateMean()) return false;
   // compute the standard deviation
-  for (int k= baseIdx; k < p_tik()->endCols(); ++k)
+  for (int k= baseIdx; k < components().end(); ++k)
   {
     p_param(k)->sigma_
     = sqrt( ( p_tik()->col(k).transpose()
