@@ -33,7 +33,7 @@
  *  @brief In this file we implement the class MixtureComposer.
  **/
 
-// will include MixtureComposer.h
+#include "../include/STK_IMixture.h"
 #include "../include/STK_MixtureComposer.h"
 #include "Arrays/include/STK_Display.h"
 
@@ -44,7 +44,8 @@ namespace STK
  *  @param nbCluster,nbSample, number of clusters and samples.
  */
 MixtureComposer::MixtureComposer( int nbSample, int nbCluster)
-                                : IMixtureComposer( nbSample, 0, nbCluster)
+                                : IMixtureComposer( nbSample, nbCluster)
+                                , meanlnLikelihood_(0.)
 { setNbFreeParameter(nbCluster-1);}
 
 /* copy constructor.
@@ -53,6 +54,7 @@ MixtureComposer::MixtureComposer( int nbSample, int nbCluster)
 MixtureComposer::MixtureComposer( MixtureComposer const& composer)
                                 : IMixtureComposer(composer)
                                 , v_mixtures_(composer.v_mixtures_)
+                                , meanlnLikelihood_(composer.meanlnLikelihood_)
 {
   // clone mixtures
   for (size_t l = 0; l < v_mixtures_.size(); ++l)
@@ -128,7 +130,8 @@ void MixtureComposer::randomInit()
   stk_cout << _T("Entering MixtureComposer::RandomInit()\n");
 #endif
   if (state() < 2) { initializeStep();}
-  randomFuzzyTik();
+  if (randomFuzzyTik()<2) throw(Clust::randomParamInitFail_);
+  mapStep();
   for (MixtIterator it = v_mixtures_.begin(); it != v_mixtures_.end(); ++it)
   { (*it)->randomInit();}
   eStep();
@@ -174,12 +177,31 @@ void MixtureComposer::samplingStep()
   { (*it)->samplingStep();}
 }
 
+/* store the  intermediate results */
 void MixtureComposer::storeIntermediateResults(int iteration)
 {
   for (MixtIterator it = v_mixtures_.begin(); it != v_mixtures_.end(); ++it)
   { (*it)->storeIntermediateResults(iteration);}
+  meanlnLikelihood_ += (lnLikelihood() - meanlnLikelihood_)/iteration;
 }
 
+void MixtureComposer::releaseIntermediateResults()
+{
+  for (MixtIterator it = v_mixtures_.begin(); it != v_mixtures_.end(); ++it)
+  { (*it)->releaseIntermediateResults();}
+  meanlnLikelihood_ = 0.;
+}
+
+/* Utility method allowing to signal to a mixture to set its parameters */
+void MixtureComposer::setParameters()
+{
+  for (MixtIterator it = v_mixtures_.begin(); it != v_mixtures_.end(); ++it)
+  { (*it)->setParameters();}
+  setLnLikelihood(meanlnLikelihood_);
+  meanlnLikelihood_ = 0.;
+}
+
+/* finalize */
 void MixtureComposer::finalizeStep()
 {
   for (MixtIterator it = v_mixtures_.begin(); it != v_mixtures_.end(); ++it)
