@@ -46,7 +46,7 @@ namespace STK
  *  @brief Compute an additive BSpline, multivalued, regression function using
  *  BSpline basis.
  */
-class AdditiveBSplineRegression : public IRegression<Matrix, Matrix, Vector>
+class AdditiveBSplineRegression : public IRegression<ArrayXX, ArrayXX, Vector>
 {
   public:
     //
@@ -59,8 +59,8 @@ class AdditiveBSplineRegression : public IRegression<Matrix, Matrix, Vector>
      * @param degree degree of the BSpline basis
      * @param position position of the knots to used
      **/
-    AdditiveBSplineRegression( Matrix const* p_y
-                             , Matrix const* p_x
+    AdditiveBSplineRegression( ArrayXX const* p_y
+                             , ArrayXX const* p_x
                              , int const& nbControlPoints
                              , int const& degree = 3
                              , KnotsPosition const& position = Regress::uniform_
@@ -80,12 +80,12 @@ class AdditiveBSplineRegression : public IRegression<Matrix, Matrix, Vector>
     /** give the control points.
      *  @return the control points of the B-Spline curves
      **/
-    inline Matrix const& controlPoints() const { return controlPoints_; }
+    inline ArrayXX const& controlPoints() const { return controlPoints_; }
     /** give the computed coefficients of the B-Spline curves.
      *   This is a matrix of size (p_x_->range(), 0:lastControlPoints).
      *  @return the coefficients of the B-Spline curve
      **/
-    inline Matrix const& coefficients() const { return coefs_.coefficients();}
+    inline ArrayXX const& coefficients() const { return coefs_.coefficients();}
 
     /** @return The extrapolated values of y from the value @c x.
      *  Given the data set @c x will compute the values \f$ y = \psi(x) \hat{\beta} \f$
@@ -93,7 +93,7 @@ class AdditiveBSplineRegression : public IRegression<Matrix, Matrix, Vector>
      *  the estimated coefficients.
      *  @param x the input data set
      **/
-    virtual Matrix extrapolate( Matrix const& x) const;
+    virtual ArrayXX extrapolate( ArrayXX const& x) const;
 
   protected:
     /** number of control points of the B-Spline curve. */
@@ -103,9 +103,9 @@ class AdditiveBSplineRegression : public IRegression<Matrix, Matrix, Vector>
     /** method of position of the knots of the B-Spline curve */
     KnotsPosition position_;
     /** Coefficients of the regression matrix */
-    AdditiveBSplineCoefficients<Matrix> coefs_;
+    AdditiveBSplineCoefficients<ArrayXX> coefs_;
     /** Estimated control points of the B-Spline curve */
-    Matrix controlPoints_;
+    ArrayXX controlPoints_;
 
     /** compute the coefficients of the BSpline basis. This method will be
      * called in the base class @c IRegression::run()
@@ -113,13 +113,13 @@ class AdditiveBSplineRegression : public IRegression<Matrix, Matrix, Vector>
     virtual void initializeStep();
 
     /** compute the regression function. */
-    virtual void regression();
+    virtual void regressionStep();
     /** compute the regression function.
      * @param weights weights of the samples
      **/
-    virtual void regression(Vector const& weights);
+    virtual void regression(VectorX const& weights);
     /** Compute the predicted outputs by the regression function. */
-    virtual void prediction();
+    virtual void predictionStep();
     /** Compute the number of parameter of the regression function.
      * @return the number of parameter of the regression function
      **/
